//@author: antokhio
//@help: template for standard shaders
//@tags: template
//@credits: 

//StructuredBuffer<float4> color <bool color=true;>;

Texture2D tex0;
SamplerState s0:IMMUTABLE {Filter=MIN_MAG_MIP_LINEAR;AddressU=CLAMP;AddressV=CLAMP;};

float4x4 tW : WORLD;
float4x4 tVP : VIEWPROJECTION;

float4 cAmb <bool color=true; String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
float4x4 tTex;


float sdSphere( float3 p, float radius ) {	return length(p) - radius; }
float sdBox( float3 p, float3 size ) {p=abs(p)-size; return max(p.x,max(p.y,p.z));} 


float sdCone( float3 p, float2 c )
{
    float q = length(p.xz);
    return dot(c,float2(q,p.y));
}

float coneAngle = 0.5;


struct VS_IN
{
	float4 pos : POSITION;
};

struct VS_OUT
{
    float4 pos: SV_POSITION;
	float4 posW: POSITION;
};

float3 p;
float3 size = 1;

float outlineWidth = 0.01;

VS_OUT VS(VS_IN input)
{
    VS_OUT output = (VS_OUT)0;
    output.pos  = mul(input.pos,mul(tW,tVP));
    output.posW = mul(input.pos,tW);
	
    return output;
}

float4 PS(VS_OUT input): SV_Target
{
	float3  pos = input.posW.xyz; // << Calculate's SDF object position against mesh
    
	float intersect = sdCone(mul (float4(pos,1),tTex).xyz, float2 (1,coneAngle)); 
	float result = abs(intersect) < outlineWidth; // If result 0 then there is intersection
	if(!result)
		discard;
	
	float4 col = cAmb * result ;
	
	return cAmb * result;// float4 (result,0,0,1); //RED LINE
}

technique10 Template
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}




