//@author: antokhio
//@help: template for standard shaders
//@tags: template
//@credits: 

float4x4 tVP : VIEWPROJECTION;	
float4x4 tW : WORLD;
float4x4 tVI:VIEWINVERSE;
float4x4 tPI:PROJECTIONINVERSE;

float4x4 tTex;

float4 cAmb <bool color=true; String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };

struct VS_IN
{
	float4 pos : POSITION;
	float4 uv : TEXCOORD0;

};

struct VS_OUT
{
    float4 pos: SV_POSITION;
    float4 uv: TEXCOORD0;
};

VS_OUT VS(VS_IN input)
{
    VS_OUT output;
    output.pos  = mul(input.pos,tW);
    output.uv= input.uv;
    return output;
}

float3 UVtoEYE(float2 UV){return normalize(mul(float4(mul(float4((UV.xy*2-1)*float2(1,-1),0,1),tPI).xy,1,0),tVI).xyz);}

float sdCone( float3 p, float2 c )
{
    float q = length(p.xz);
    return dot(c,float2(q,p.y));
}

float coneAngle = 0.5;

float f (float3 p)
{
	//p.xz = 1 - frac (p.xz)*2;
	float d = 99999;
	//d = max(abs(p.x),abs (p.z));
	d = min (d, sdCone(mul (float4(p,1),tTex).xyz, float2 (1,coneAngle)));
	
	return d;
}

float trace (in float3 ro, in float3 rd, inout float3 p)
{
	float t = 0.0;
	for (int i = 0 ; i < 128 ; i++)
	{
		p = ro + rd*t;
		float d = f(p);
		t += d * 0.5;
	}
	return t;
}

float3 calcNormal( in float3 pos )
{
	float3 eps = float3( 0.0001, 0.0, 0.0 );
	float3 nor = float3(
	    f(pos+eps.xyy) - f(pos-eps.xyy),
	    f(pos+eps.yxy) - f(pos-eps.yxy),
	    f(pos+eps.yyx) - f(pos-eps.yyx) );
	return normalize(nor);
}

struct PS_OUT
{
	float4 color : SV_Target0;
	float depth : SV_Depth;
};


PS_OUT PS(VS_OUT input)
{
    PS_OUT Out;
	float3 p;

	float3 ro = tVI[3].xyz;
	float3 rd = UVtoEYE(input.uv.xy);
	
	float d = trace (ro, rd, p);
	
	float fog = 1/(1 + d * d *0.25);
		
	float4 pos = mul (float4 (p.xyz,1),tVP);
	
	if (abs(f(p)) > 0.01) discard;
	
	Out.color = fog * cAmb;
	Out.depth = pos.z/pos.w;
	
    return Out;
}





technique10 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}




